==> UnderB-1.01.tar.gz: UnderB-1.01/vote.c <==
#define	VOTE_CONTROL	"vote/control"
#define	VOTE_DESC	"vote/desc"
#define	VOTE_BALLOT	"vote/ballots"
#define	VOTE_RESULT	"vote/results"
#define	VOTE_NRSEULT	"vote/newresults"
#define	VOTE_TEMP	"vote/temp"

int suckinfile(fp, fname)
FILE *fp;
char *fname;
{
  char inbuf[256];
  FILE *sfp;
  if ((sfp = fopen(fname, "r")) == NULL) return -1;
  while (fgets(inbuf, sizeof(inbuf), sfp) != NULL)
    fputs(inbuf, fp);
  fclose(sfp);
  return 0;
}

int Vote()
{
  FILE *fp;
  int rc, fd, count = 0;
  time_t closetime;
  char inbuf[81], choices[10], vote[2];
  struct stat statb;
  extern int lang;
  extern USERDATA user_params;

  move(3, 0);
  if ((fp = fopen(VOTE_CONTROL, "r")) == NULL)
    {
      if (lang)
	prints("ǥ  Ǿ  ʳ׿..\n");
      else
	prints("Sorry, the polls are not open at this time.\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }
  if (my_flag(FLG_VOTE))
    {
      if (lang)
	prints("̹ ǥ ߳׿.. ѹۿ ǥ մϴ.\n");
      else
	prints("You've already Voted!\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }
  bbs_set_mode (M_VOTE);

  More (VOTE_DESC, 1);

  clear();
  move(0, 0);
  standout();
  if (lang)
    prints("ǥ");
  else
    prints("THE BALLOT");
  standend();

  if (lang)
    {
      prints("To cast your vote, enter the digit next to your selection.\n");
    }
  else
    {
      prints("  ȣ \n");
    }

  /* get the closing time */
  fgets(inbuf, sizeof(inbuf), fp);
  closetime = (time_t) atoi(inbuf);
  if (lang)
    prints("ǥ ð: %s\n", ctime(&closetime));
  else
    prints("This poll will close at: %s\n", ctime(&closetime));
  move (6, 0);
  memset(choices, '\0', sizeof(choices));
  while (fgets(inbuf, sizeof(inbuf), fp) != NULL)
    {
      choices[count++] = inbuf[0];
      prints("%s", inbuf);
    }
  fclose(fp);

  memset(vote, '\0', sizeof(vote));
  if (lang)
    getdata(count+8, 0, ": ", vote, 2, DOECHO, NULL);
  else
    getdata(count+8, 0, "Enter your choice: ", vote, 2, DOECHO, NULL);

  move (count+10, 0);

  if (vote[0] == '\0' || index(choices, vote[0]) == NULL)
    {
      if (lang)
	prints("߸ Դϴ. ǥ ߽ϴ.\n");
      else
	prints("Invalid choice.. voting aborted.\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }

  if ((fd = open(VOTE_BALLOT, O_WRONLY|O_CREAT|O_APPEND, 0600)) < 0)
    {
      if (lang)
	prints("ǥ ȿ׿...  ǥԴϴ.\n");
      else
	prints("Cannot open the ballot box! Aborting..\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }

  flock(fd, LOCK_EX);
  write(fd, vote, 1);
  flock(fd, LOCK_UN);
  fstat(fd, &statb);
  close(fd);

  utable_get_record(my_userid(), &user_params);
  user_params.u.flags |= FLG_VOTE;
  utable_set_record(my_userid(), &user_params);

  if (lang)
    prints("ǥ ߽ϴ. ( %d  ǥ)\n", statb.st_size);
  else
    prints("Ballot recorded! (%d cast so far)\n", statb.st_size);
  pressreturn();
  bbs_set_mode(M_UNDEFINED);
  return (FULLUPDATE);
}

int Result()
{
  extern int lang;

  if (More(VOTE_RESULT, 1) == -1)
    {
      move(3,0);
      if (lang)
	prints("ǥ ׿..\n");
      else
	prints("No vote results are available at this time.\n");
      pressreturn;
    }
  return (FULLUPDATE);
}

int OpenVote()
{
  FILE *fp;
  struct stat statb;
  char inbuf[80];
  char *newline;
  int numselections = 0, aborted;
  time_t closetime, numseconds;

  extern USERDATA user_params;
  extern int lang;

  clear();
  move(0,0);
  standout();
  if (lang)
    prints("ǥ !");
  else
    prints("Open the Polls!");
  standend();
  move (2,0);
  if ((fp = fopen(VOTE_CONTROL, "r")) != NULL)
    {
      fgets(inbuf, sizeof(inbuf), fp);
      fclose(fp);
      if (newline = index(inbuf, '\n')) *newline = '\0';
      closetime = atof(inbuf);

      if (lang)
	{
	  prints("̹ ǥ Ǿ ֽϴ.\n");
	  prints(" ð: %s", ctime(&closetime));
	}
      else
	{
	  prints("Sorry, the polls are already open.\n");
	  prints("They are scheduled to close at: %s", ctime(&closetime));
	}
      pressreturn();
      clear();
      return FULLUPDATE;
    }

  if (Edit(VOTE_DESC))
    {
      clear();
      if (lang)
	prints("ǥ  ߽ϴ.\n");
      else
	prints("Open polls ABORTED\n");
      pressreturn();
      clear();
      return FULLUPDATE;
    }

  fp = fopen(VOTE_CONTROL, "w");
  clear();
  if (lang)
    getdata(0, 0, "ĥ  ǥұ? : ", inbuf, 4, DOECHO, NULL);
  else
    getdata(0, 0, "Number of days to keep polls open : ", inbuf, 4, DOECHO, NULL);
  numseconds = atoi(inbuf) * 86400;
  time (&closetime);
  closetime += numseconds;
  fprints(fp, "%lu\n", closetime);
  fflush(fp);

  while (!aborted) {
    getdata(numselections+1,0,"Enter selection (RETURN if done): ",
           inbuf, 40, DOECHO, NULL);
    if (*inbuf) {           
      if (numselections == 9) numselections = 0;
      else numselections++;
      fprintf(fp, "%1d) %s\n", numselections, inbuf);
    }
    if (*inbuf == '\0' || numselections == 0)
      aborted = 1;
  }
  fclose(fp);

  utable_get_record(my_userid(), &user_params);
  user_params.u.flags &= ~FLG_VOTE;
  utable_set_record(my_userid(), &user_params);

  prints("The polls are open!\n");
  pressreturn();
  return FULLUPDATE;
}

closepolls()
{
        FILE *cfp, *tfp;
        char inchar, inbuf[80];
        int bfd, counts[10];
        time_t endtime;
        if ((cfp = fopen(VOTE_CONTROL, "r")) == NULL) {
            if (currentuser.flags[0] & VOTE_FLAG) {
                currentuser.flags[0] &= ~VOTE_FLAG;
                substitute_record(PASSFILE, &currentuser, sizeof(currentuser), u
            }
            return 0;
        }
        if (fgets(inbuf, sizeof(inbuf), cfp) == NULL) {
            fclose(cfp);
            return;
        }
        fclose(cfp);
        endtime = (time_t)atol(inbuf);
        if (endtime > time(0) || endtime <= 0) return 0;
        if (rename(VOTE_CONTROL, VOTE_TEMP) == -1) return 0;
        report("closed the polls");
        for (bfd = 0; bfd < 10; bfd++) counts[bfd] = 0;
        if ((bfd = open(VOTE_BALLOT, O_RDONLY)) != -1) {
            while (read(bfd, &inchar, 1) == 1) 
                counts[(int)(inchar - '0')]++;
            close(bfd);
        }
        if (tfp = fopen(VOTE_NRESULT, "w")) {
            int totvotes = 0;
            fprintf(tfp, "------------------------------------------------------
            fprintf(tfp, "** Polls closed: %s\n", ctime(&endtime));
            fprintf(tfp, "** Synopsis of ballot issue:\n\n");
            suckinfile(tfp, VOTE_DESC);
            fprintf(tfp, "** Results:\n\n");
            if (cfp = fopen(VOTE_TEMP, "r")) {
                int cindex;
                fgets(inbuf, sizeof(inbuf), cfp);
                while (fgets(inbuf, sizeof(inbuf), cfp)) {
                    inbuf[(strlen(inbuf)-1)] = '\0';
                    inbuf[23] = '\0'; /* truncate */
                    cindex = inbuf[0] - '0';                
                    fprintf(tfp, "    %-20s  %3d\n", inbuf+3, counts[cindex]);
                    totvotes += counts[cindex];
                }
                fclose(cfp);
            }
            fprintf(tfp, "\nTotal votes cast = %d\n\n", totvotes);
            suckinfile(tfp, VOTE_RESULT);
            fclose(tfp);
        }
        else return -1; /* couldn't write out the results! */
        unlink(VOTE_RESULT);
        rename(VOTE_NRESULT, VOTE_RESULT);
        unlink(VOTE_RESC);
        unlink(VOTE_TEMP);
        unlink(VOTE_BALLOT);
        clearflags();
        currentuser.flags[0] &= ~VOTE_FLAG;
        return 0;
}

==> UnderB-1.091.tar.gz: UnderB-1.091/vote.c <==
#define	VOTE_CONTROL	"vote/control"
#define	VOTE_DESC	"vote/desc"
#define	VOTE_BALLOT	"vote/ballots"
#define	VOTE_RESULT	"vote/results"
#define	VOTE_NRSEULT	"vote/newresults"
#define	VOTE_TEMP	"vote/temp"

int suckinfile(fp, fname)
FILE *fp;
char *fname;
{
  char inbuf[256];
  FILE *sfp;
  if ((sfp = fopen(fname, "r")) == NULL) return -1;
  while (fgets(inbuf, sizeof(inbuf), sfp) != NULL)
    fputs(inbuf, fp);
  fclose(sfp);
  return 0;
}

int Vote()
{
  FILE *fp;
  int rc, fd, count = 0;
  time_t closetime;
  char inbuf[81], choices[10], vote[2];
  struct stat statb;
  extern int lang;
  extern USERDATA user_params;

  move(3, 0);
  if ((fp = fopen(VOTE_CONTROL, "r")) == NULL)
    {
      if (lang)
	prints("ǥ  Ǿ  ʳ׿..\n");
      else
	prints("Sorry, the polls are not open at this time.\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }
  if (my_flag(FLG_VOTE))
    {
      if (lang)
	prints("̹ ǥ ߳׿.. ѹۿ ǥ մϴ.\n");
      else
	prints("You've already Voted!\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }
  bbs_set_mode (M_VOTE);

  More (VOTE_DESC, 1);

  clear();
  move(0, 0);
  standout();
  if (lang)
    prints("ǥ");
  else
    prints("THE BALLOT");
  standend();

  if (lang)
    {
      prints("To cast your vote, enter the digit next to your selection.\n");
    }
  else
    {
      prints("  ȣ \n");
    }

  /* get the closing time */
  fgets(inbuf, sizeof(inbuf), fp);
  closetime = (time_t) atoi(inbuf);
  if (lang)
    prints("ǥ ð: %s\n", ctime(&closetime));
  else
    prints("This poll will close at: %s\n", ctime(&closetime));
  move (6, 0);
  memset(choices, '\0', sizeof(choices));
  while (fgets(inbuf, sizeof(inbuf), fp) != NULL)
    {
      choices[count++] = inbuf[0];
      prints("%s", inbuf);
    }
  fclose(fp);

  memset(vote, '\0', sizeof(vote));
  if (lang)
    getdata(count+8, 0, ": ", vote, 2, DOECHO, NULL);
  else
    getdata(count+8, 0, "Enter your choice: ", vote, 2, DOECHO, NULL);

  move (count+10, 0);

  if (vote[0] == '\0' || index(choices, vote[0]) == NULL)
    {
      if (lang)
	prints("߸ Դϴ. ǥ ߽ϴ.\n");
      else
	prints("Invalid choice.. voting aborted.\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }

  if ((fd = open(VOTE_BALLOT, O_WRONLY|O_CREAT|O_APPEND, 0600)) < 0)
    {
      if (lang)
	prints("ǥ ȿ׿...  ǥԴϴ.\n");
      else
	prints("Cannot open the ballot box! Aborting..\n");
      clrtobot();
      pressreturn();
      return (FULLUPDATE);
    }

  flock(fd, LOCK_EX);
  write(fd, vote, 1);
  flock(fd, LOCK_UN);
  fstat(fd, &statb);
  close(fd);

  utable_get_record(my_userid(), &user_params);
  user_params.u.flags |= FLG_VOTE;
  utable_set_record(my_userid(), &user_params);

  if (lang)
    prints("ǥ ߽ϴ. ( %d  ǥ)\n", statb.st_size);
  else
    prints("Ballot recorded! (%d cast so far)\n", statb.st_size);
  pressreturn();
  bbs_set_mode(M_UNDEFINED);
  return (FULLUPDATE);
}

int Result()
{
  extern int lang;

  if (More(VOTE_RESULT, 1) == -1)
    {
      move(3,0);
      if (lang)
	prints("ǥ ׿..\n");
      else
	prints("No vote results are available at this time.\n");
      pressreturn;
    }
  return (FULLUPDATE);
}

int OpenVote()
{
  FILE *fp;
  struct stat statb;
  char inbuf[80];
  char *newline;
  int numselections = 0, aborted;
  time_t closetime, numseconds;

  extern USERDATA user_params;
  extern int lang;

  clear();
  move(0,0);
  standout();
  if (lang)
    prints("ǥ !");
  else
    prints("Open the Polls!");
  standend();
  move (2,0);
  if ((fp = fopen(VOTE_CONTROL, "r")) != NULL)
    {
      fgets(inbuf, sizeof(inbuf), fp);
      fclose(fp);
      if (newline = index(inbuf, '\n')) *newline = '\0';
      closetime = atof(inbuf);

      if (lang)
	{
	  prints("̹ ǥ Ǿ ֽϴ.\n");
	  prints(" ð: %s", ctime(&closetime));
	}
      else
	{
	  prints("Sorry, the polls are already open.\n");
	  prints("They are scheduled to close at: %s", ctime(&closetime));
	}
      pressreturn();
      clear();
      return FULLUPDATE;
    }

  if (Edit(VOTE_DESC))
    {
      clear();
      if (lang)
	prints("ǥ  ߽ϴ.\n");
      else
	prints("Open polls ABORTED\n");
      pressreturn();
      clear();
      return FULLUPDATE;
    }

  fp = fopen(VOTE_CONTROL, "w");
  clear();
  if (lang)
    getdata(0, 0, "ĥ  ǥұ? : ", inbuf, 4, DOECHO, NULL);
  else
    getdata(0, 0, "Number of days to keep polls open : ", inbuf, 4, DOECHO, NULL);
  numseconds = atoi(inbuf) * 86400;
  time (&closetime);
  closetime += numseconds;
  fprints(fp, "%lu\n", closetime);
  fflush(fp);

  while (!aborted) {
    getdata(numselections+1,0,"Enter selection (RETURN if done): ",
           inbuf, 40, DOECHO, NULL);
    if (*inbuf) {           
      if (numselections == 9) numselections = 0;
      else numselections++;
      fprintf(fp, "%1d) %s\n", numselections, inbuf);
    }
    if (*inbuf == '\0' || numselections == 0)
      aborted = 1;
  }
  fclose(fp);

  utable_get_record(my_userid(), &user_params);
  user_params.u.flags &= ~FLG_VOTE;
  utable_set_record(my_userid(), &user_params);

  prints("The polls are open!\n");
  pressreturn();
  return FULLUPDATE;
}

closepolls()
{
        FILE *cfp, *tfp;
        char inchar, inbuf[80];
        int bfd, counts[10];
        time_t endtime;
        if ((cfp = fopen(VOTE_CONTROL, "r")) == NULL) {
            if (currentuser.flags[0] & VOTE_FLAG) {
                currentuser.flags[0] &= ~VOTE_FLAG;
                substitute_record(PASSFILE, &currentuser, sizeof(currentuser), u
            }
            return 0;
        }
        if (fgets(inbuf, sizeof(inbuf), cfp) == NULL) {
            fclose(cfp);
            return;
        }
        fclose(cfp);
        endtime = (time_t)atol(inbuf);
        if (endtime > time(0) || endtime <= 0) return 0;
        if (rename(VOTE_CONTROL, VOTE_TEMP) == -1) return 0;
        report("closed the polls");
        for (bfd = 0; bfd < 10; bfd++) counts[bfd] = 0;
        if ((bfd = open(VOTE_BALLOT, O_RDONLY)) != -1) {
            while (read(bfd, &inchar, 1) == 1) 
                counts[(int)(inchar - '0')]++;
            close(bfd);
        }
        if (tfp = fopen(VOTE_NRESULT, "w")) {
            int totvotes = 0;
            fprintf(tfp,
"--------------------------------------------------------------------------"};
            fprintf(tfp, "** Polls closed: %s\n", ctime(&endtime));
            fprintf(tfp, "** Synopsis of ballot issue:\n\n");
            suckinfile(tfp, VOTE_DESC);
            fprintf(tfp, "** Results:\n\n");
            if (cfp = fopen(VOTE_TEMP, "r")) {
                int cindex;
                fgets(inbuf, sizeof(inbuf), cfp);
                while (fgets(inbuf, sizeof(inbuf), cfp)) {
                    inbuf[(strlen(inbuf)-1)] = '\0';
                    inbuf[23] = '\0'; /* truncate */
                    cindex = inbuf[0] - '0';                
                    fprintf(tfp, "    %-20.20s  %3d\n", inbuf+3, counts[cindex]);
                    totvotes += counts[cindex];
                }
                fclose(cfp);
            }
            fprintf(tfp, "\nTotal votes cast = %d\n\n", totvotes);
            suckinfile(tfp, VOTE_RESULT);
            fclose(tfp);
        }
        else return -1; /* couldn't write out the results! */
        unlink(VOTE_RESULT);
        rename(VOTE_NRESULT, VOTE_RESULT);
        unlink(VOTE_RESC);
        unlink(VOTE_TEMP);
        unlink(VOTE_BALLOT);
        clearflags();
        currentuser.flags[0] &= ~VOTE_FLAG;
        return 0;
}
